// Library     : DPMI
// Description : DOS Protected Mode Interface function library
// Module      : DPMI02.C
// Version     : 1.00 - 01/09/1994
// Written by  : David A. Mair
// -----------------------------------------------------------------------------
// Change summary --------------------------------------------------------------
// -----------------------------------------------------------------------------
//    Date   | Version | Comment and author
// -----------------------------------------------------------------------------
//           |         |
//           |         |
// -----------------------------------------------------------------------------


// Inline assembler is included in this module
#pragma inline

#include "dpmi.h"


REAL_MODE_FAR_POINTER GetRealModeInterruptVector(BYTE byIntNum)
{
	REAL_MODE_FAR_POINTER rmfpRetVal;

	asm {
		mov		ax, 0x0200
		mov		bl, byte ptr [byIntNum]
		int		0x31

		mov		word ptr [rmfpRetVal], dx
		mov		word ptr [rmfpRetVal + 2], cx
	}

	return rmfpRetVal;
}


void SetRealModeInterruptVector(BYTE byIntNum, REAL_MODE_FAR_POINTER rmfpNewVector)
{
	asm {
		mov		ax, 0x0201
		mov		bl, byte ptr [byIntNum]
		mov		dx, word ptr [rmfpNewVector]
		mov		cx, word ptr [rmfpNewVector + 2]
		int		0x31
	}
}


#ifdef __DPMI32__
WORD GetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector)
#else
WORD GetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0202
		mov		bl, byte ptr [byExceptionNum]
		int		0x31

		jc		gpmehvSetRetVal

#ifdef __DPMI32__
		mov		ebx, dword ptr [pmfp32ExceptionVector]
		mov		dword ptr [ebx], edx
		mov		word ptr [ebx + 4], cx
#else
		push	es
		les		bx, pmfp16ExceptionVector
		mov		word ptr es:[bx], dx
		mov		word ptr es:[bx + 2], cx
		pop		es
#endif

		mov		ax, 0x0000
	}

gpmehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
WORD SetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector)
#else
WORD SetProcessorExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0203
		mov		bl, byte ptr [byExceptionNum]
#ifdef __DPMI32__
		mov		edx, dword ptr [pmfp32NewVector]
		mov		cx, word ptr [pmfp32NewVector + 4]
#else
		mov		dx, word ptr [pmfp16NewVector]
		mov		cx, word ptr [pmfp16NewVector + 2]
#endif
		int		0x31

		jc		spehvSetRetVal

		mov		ax, 0x0000
	}

spehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
PROT_MODE_FAR_POINTER_32 GetProtectedModeInterruptVector(BYTE byIntNum)
#else
PROT_MODE_FAR_POINTER_16 GetProtectedModeInterruptVector(BYTE byIntNum)
#endif
{
#ifdef __DPMI32__
	PROT_MODE_FAR_POINTER_32 pmfp32RetVal;
#else
	PROT_MODE_FAR_POINTER_16 pmfp16RetVal;
#endif

	asm {
		mov		ax, 0x0204
		mov		bl, byte ptr [byIntNum]
		int		0x31

#ifdef __DPMI32__
		mov		dword ptr [pmfp32RetVal], edx
		mov		word ptr [pmfp32RetVal + 4], cx
#else
		mov		word ptr [pmfp16RetVal], dx
		mov		word ptr [pmfp16RetVal + 2], cx
#endif
	}

#ifdef __DPMI32__
	return pmfp32RetVal;
#else
	return pmfp16RetVal;
#endif
}


#ifdef __DPMI32__
WORD SetProtectedModeInterruptVector(BYTE byIntNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector)
#else
WORD SetProtectedModeInterruptVector(BYTE byIntNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0205
		mov		bl, byte ptr [byIntNum]
#ifdef __DPMI32__
		mov		edx, dword ptr [pmfp32NewVector]
		mov		cx, word ptr [pmfp32NewVector + 2]
#else
		mov		dx, word ptr [pmfp16NewVector]
		mov		cx, word ptr [pmfp16NewVector]
#endif
		int		0x31

		jc		spmivSetRetVal

		mov		ax, 0x0000
	}

spmivSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}



#ifdef __DPMI32__
WORD GetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector)
#else
WORD GetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0210
		mov		bl, byte ptr [byExceptionNum]
		int		0x31

		jc		geppmehvSetRetVal

#ifdef __DPMI32__
		mov		ebx, dword ptr [pmfp32ExceptionVector]
		mov		dword ptr [ebx], edx
		mov		word ptr [ebx + 4], cx
#else
		push	es
		les		bx, pmfp16ExceptionVector
		mov		word ptr es:[bx], dx
		mov		word ptr es:[bx + 4], cx
		pop		es
#endif
		mov		ax, 0x0000
	}

geppmehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
WORD GetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 *pmfp32ExceptionVector)
#else
WORD GetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 far *pmfp16ExceptionVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0211
		mov		bl, byte ptr [byExceptionNum]
		int		0x31

		jc		geprmehvSetRetVal

#ifdef __DPMI32__
		mov		ebx, dword ptr [pmfp32ExceptionVector]
		mov		dword ptr [ebx], edx
		mov		word ptr [ebx + 4], cx
#else
		push	es
		les		bx, pmfp16ExceptionVector
		mov		word ptr es:[bx], dx
		mov		word ptr es:[bx + 4], cx
		pop		es
#endif
		mov		ax, 0x0000
	}

geprmehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
WORD SetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector)
#else
WORD SetExtendedProcessorProtModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0212
		mov		bl, byte ptr [byExceptionNum]
#ifdef __DPMI32__
		mov		edx, dword ptr [pmfp32NewVector]
		mov		cx, word ptr [pmfp32NewVector + 4]
#else
		mov		dx, word ptr [pmfp16NewVector]
		mov		cx, word ptr [pmfp16NewVector + 2]
#endif
		int		0x31

		jc		seppmehvSetRetVal

		mov		ax, 0x0000
	}

seppmehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


#ifdef __DPMI32__
WORD SetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_32 pmfp32NewVector)
#else
WORD SetExtendedProcessorRealModeExceptionHandlerVector(BYTE byExceptionNum, PROT_MODE_FAR_POINTER_16 pmfp16NewVector)
#endif
{
	WORD wRetVal;

	asm {
		mov		ax, 0x0213
		mov		bl, byte ptr [byExceptionNum]
#ifdef __DPMI32__
		mov		edx, dword ptr [pmfp32NewVector]
		mov		cx, word ptr [pmfp32NewVector + 4]
#else
		mov		dx, word ptr [pmfp16NewVector]
		mov		cx, word ptr [pmfp16NewVector + 2]
#endif
		int		0x31

		jc		seprmehvSetRetVal

		mov		ax, 0x0000
	}

seprmehvSetRetVal:
	asm {
		mov		word ptr [wRetVal], ax
	}

	return wRetVal;
}


